import { Metadata, FormatterParserProvider, Http, Validation } from '@zeedhi/core';
import { Modal, ModalService, Grid, Form, LoadingService, AlertService, DialogService } from '@zeedhi/common';

//  Formata texto da coluna Periodo
FormatterParserProvider.registerFormatter('PeriodoFormatter', (periodObject: any) => "De " + periodObject.start + " até " + periodObject.end);
//  Funciona como um ZdSelectMultiple, iterando pelos valores e chamando ZdSelect
FormatterParserProvider.registerFormatter('column_SelectMultipleFormatter', (params: any) => {
    const formatterFn = FormatterParserProvider.getFormatter('column_ZdSelect');
    if (params.value?.length > 1) {
        const values = params.value.map((value: any) => formatterFn({...params, value: [value]}));
        return values.join(', ');
    }
    return formatterFn(params);
});
//  Validação do Date Range
const newValidation = (config: any) => (value: string[]) => value.length !== 1 || config.message || "Duas datas devem ser apresentadas";
Validation.register('dateRangeValidation', newValidation);
//  Controller
export class PeriodObservationController {
    //  Metodos de alert/dialog
    public showAlert(type: string, mensagem: string) {
        AlertService.show({
            name: 'custom-alert',
            text: mensagem,
            right: true,
            top: true,
            color: type
        });
    }
    public showErrorAlert() {
        this.showAlert("error", "Ocorreu um erro!");
    }
    public dialogEvents: any = {
        click: async ({ component }: any) => {
            if (component.name === "sim") {
                await this.deleteRow();
            }
            DialogService.hide();
        }
    };
    public showDeleteDialog({ event }: any) {
        event.preventDefault();
        DialogService.show({
          name: 'delete-dialog',
          type: 'warning',
          title: ' ',
          text: 'Deseja remover o(s) registro(s) selecionado(s)?',
          buttons: [
            { name: 'sim', component: 'ZdButton', label: 'SIM', events: this.dialogEvents, outline: true },
            { name: 'não', component: 'ZdButton', label: 'NÃO', events: this.dialogEvents },
          ],
        });
    }
    //  Metodos de Modal
    private detailModal!: Modal;
    public async createDetailModal() {
        this.detailModal = await ModalService.createFromJson('periodObservation/periodDetailModal', true);
    }
    public destroyDetailModal() {
        this.detailModal?.destroy();
    }
    public closeDetailModal() {
        this.detailModal.hide();
    }
    //  Metodos do Formulario
    protected modalData: any;
    public get currentRowId() {
        return this.modalData._id;
    }
    public get modalTitle() {
        return (this.currentRowId ? "Editar" : "Adicionar") + " Observação de período";
    }
    public formOnMounted({ component }: any) {
        const form = component;
        const data = this.modalData;
        form.getChildInstance('name').value = data.name;
        form.getChildInstance('period').setDateValue(data.period.start + ' - ' + data.period.end);
        form.getChildInstance('indicator').setValue(data.indicator);
        form.getChildInstance('dataSource').setValue(data.dataSource);
    }
    public async openDetailModal({ component, row }: any) {
        this.destroyDetailModal();
        await this.createDetailModal();
        const defaultValues = {name: "", period: "", indicator: [], dataSource: []};
        this.modalData = row === undefined ? defaultValues : row;
        this.detailModal.show();
    }
    public saveDetailModal() {
        const form = this.detailModal.getChildInstance<Form>("detailForm");
        const values = form.value;
        if (!form.validate()) {
            return;
        }
        const content = {
            name: values.name,
            period: {start: values.period[0], end: values.period[1]},
            indicator: values.indicator?.length > 0 ? values.indicator : null,
            dataSource: values.dataSource?.length > 0 ? values.dataSource : null,
            _id: this.currentRowId,
            "changed": true
        };
        const addContent = {
            __createdLocal: true,
            __is_new: true,
            ...content
        };
        Http.post("periodObservation/save", {
            "requestType": "DataSet",
            "disableLoader": false,
            "dataset": [this.currentRowId ? content : addContent]
        }).then(() => {
            this.showAlert('success', "Salvo com sucesso!");
        }).catch(() => {
            this.showErrorAlert();
        }).finally(() => {
            this.reload();
            this.closeDetailModal();
        });
    }
    //  Grid Methods
    public gridOnMounted() {
        const botaoRefreshGrid: any = Metadata.getInstance("period-observation-grid_refreshButton");
        botaoRefreshGrid.events.click = this.reload;
        this.reload();
    }
    public deleteRow() {
        const grid = Metadata.getInstance<Grid>('period-observation-grid');
        return Http.post("/periodObservation/delete", {
            "requestType": "Row",
            "row": grid.selectedRows
        }).then(() => {
            this.showAlert('success', "Deletado com sucesso!");
        }).catch(() => {
            this.showErrorAlert();
        }).finally(() => {
            this.reload();
            grid.selectedRows = [];
        });
    }
    public async reload() {
        //  Os códigos a seguir tem unicamente o proposito de apresentar Loading enquanto os dados de
        LoadingService.show("Aguarde...");
        const grid = Metadata.getInstance<Grid>('period-observation-grid');
        grid.isVisible = false;
        const indicatorRequest = Http.get("indicator");
        const dataSourceRequest = Http.get("dataSource");
        return Promise.all([indicatorRequest, dataSourceRequest, grid.datasource.get()]).catch(console.error).finally(() => {
            LoadingService.hide();
            grid.isVisible = true;
        });
    }
}
